package SPVM::Error::Unicode::InvalidUTF8;



1;

=head1 Name

SPVM::Error::Unicode::InvalidUTF8 - Invalid UTF8 Errors

=head1 Description

The Error::Unicode::InvalidUTF8 class of L<SPVM> represents Invalid UTF8 errors

=head1 Usage

  use Error::Unicode::InvalidUTF8;

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License

