package SPVM::Error::Compile;

1;

=head1 Name

SPVM::Error::Compile - Compilation Errors

=head1 Usage

  use Error::Compile;

=head1 Description

C<Error::Compile> is the error class for compilation errors.

=head1 Super Class

L<Error|SPVM::Error>

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License
