=head1 Name

SPVM::Document::Modules - SPVM Standard Modules

=head1 Description

This document lists standard modules in the SPVM language. Standard modules are the modules contained in the distribution of the SPVM language.

=head1 Standard Modules

=over 2

=item * L<Address|SPVM::Address>

=item * L<Array|SPVM::Array>

=item * L<Bool|SPVM::Bool>

=item * L<Byte|SPVM::Byte>

=item * L<ByteList|SPVM::ByteList>

=item * L<Callback|SPVM::Callback>

=item * L<Callback::Grep|SPVM::Callback::Grep>

=item * L<Callback::Map|SPVM::Callback::Map>

=item * L<Callback::MapExpand|SPVM::Callback::MapExpand>

=item * L<Cloneable|SPVM::Cloneable>

=item * L<Cloner|SPVM::Cloner>

=item * L<CommandInfo|SPVM::CommandInfo>

=item * L<Comparable|SPVM::Comparable>

=item * L<Comparator|SPVM::Comparator>

=item * L<Comparator::Double|SPVM::Comparator::Double>

=item * L<Comparator::Float|SPVM::Comparator::Float>

=item * L<Comparator::Int|SPVM::Comparator::Int>

=item * L<Comparator::Long|SPVM::Comparator::Long>

=item * L<Comparator::String|SPVM::Comparator::String>

=item * L<Complex_2d|SPVM::Complex_2d>

=item * L<Complex_2f|SPVM::Complex_2f>

=item * L<Double|SPVM::Double>

=item * L<DoubleList|SPVM::DoubleList>

=item * L<EqualityChecker|SPVM::EqualityChecker>

=item * L<EqualityChecker::Address|SPVM::EqualityChecker::Address>

=item * L<EqualityCheckable|SPVM::EqualityCheckable>

=item * L<Error|SPVM::Error>

=item * L<Error::Compile|SPVM::Error::Compile>

=item * L<Error::NotSupported|SPVM::Error::NotSupported>

=item * L<Error::System|SPVM::Error::System>

=item * L<Error::Unicode::InvalidUTF8|SPVM::Error::Unicode::InvalidUTF8>

=item * L<Float|SPVM::Float>

=item * L<FloatList|SPVM::FloatList>

=item * L<Fn|SPVM::Fn>

=item * L<Format|SPVM::Format>

=item * L<Hash|SPVM::Hash>

=item * L<Hash::Entry|SPVM::Hash::Entry>

=item * L<Immutable::ByteList|SPVM::Immutable::ByteList>

=item * L<Immutable::DoubleList|SPVM::Immutable::DoubleList>

=item * L<Immutable::FloatList|SPVM::Immutable::FloatList>

=item * L<Immutable::IntList|SPVM::Immutable::IntList>

=item * L<Immutable::LongList|SPVM::Immutable::LongList>

=item * L<Immutable::ShortList|SPVM::Immutable::ShortList>

=item * L<Immutable::StringList|SPVM::Immutable::StringList>

=item * L<Int|SPVM::Int>

=item * L<IntList|SPVM::IntList>

=item * L<List|SPVM::List>

=item * L<Long|SPVM::Long>

=item * L<LongList|SPVM::LongList>

=item * L<Native|SPVM::Native>

=item * L<Native::Arg|SPVM::Native::Arg>

=item * L<Native::BasicType|SPVM::Native::BasicType>

=item * L<Native::ClassFile|SPVM::Native::ClassFile>

=item * L<Native::ClassVar|SPVM::Native::ClassVar>

=item * L<Native::Compiler|SPVM::Native::Compiler>

=item * L<Native::Env|SPVM::Native::Env>

=item * L<Native::Field|SPVM::Native::Field>

=item * L<Native::Method|SPVM::Native::Method>

=item * L<Native::MethodCall|SPVM::Native::MethodCall>

=item * L<Native::Runtime|SPVM::Native::Runtime>

=item * L<Native::Stack|SPVM::Native::Stack>

=item * L<Point|SPVM::Point>

=item * L<Point3D|SPVM::Point3D>

=item * L<Scope::Guard|SPVM::Scope::Guard>

=item * L<Short|SPVM::Short>

=item * L<ShortList|SPVM::ShortList>

=item * L<Sort|SPVM::Sort>

=item * L<SPVM|SPVM::SPVM>

=item * L<Stringable|SPVM::Stringable>

=item * L<StringBuffer|SPVM::StringBuffer>

=item * L<Stringer|SPVM::Stringer>

=item * L<StringList|SPVM::StringList>

=item * L<Sync::Mutex|SPVM::Sync::Mutex>

=back

=head1 CPAN Modules

L<CPAN Modules - SPVM Wiki|https://github.com/yuki-kimoto/SPVM/wiki/CPAN-Modules>

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License
